@@ -3,9 +3,10 @@ from __future__ import annotations
 import pathlib
 from typing import Union
 
+from ada import fem
 from ada.concepts.connections import Bolts, Weld
 from ada.concepts.curves import ArcSegment, CurvePoly, CurveRevolve, LineSegment
-from ada.concepts.levels import FEM, Assembly, Part
+from ada.concepts.levels import Assembly, Part
 from ada.concepts.piping import Pipe, PipeSegElbow, PipeSegStraight
 from ada.concepts.points import Node
 from ada.concepts.primitives import (
@@ -18,8 +19,12 @@ from ada.concepts.primitives import (
     PrimSweep,
     Shape,
 )
-from ada.concepts.structural import Beam, Plate, Wall
+from ada.concepts.stru_beams import Beam
+from ada.concepts.stru_plates import Plate
+from ada.concepts.stru_walls import Wall
+from ada.concepts.transforms import Placement, Transform
 from ada.config import User
+from ada.fem import FEM
 from ada.materials import Material
 from ada.sections import Section
 
@@ -43,15 +48,17 @@ def from_fem(
     fem_format: Union[str, list] = None,
     name: Union[str, list] = None,
     enable_experimental_cache=False,
+    source_units="m",
+    fem_converter="default",
 ) -> Assembly:
-    a = Assembly(enable_experimental_cache=enable_experimental_cache)
-    if type(fem_file) in (str, pathlib.WindowsPath):
-        a.read_fem(fem_file, fem_format, name)
+    a = Assembly(enable_experimental_cache=enable_experimental_cache, units=source_units)
+    if type(fem_file) is str or issubclass(type(fem_file), pathlib.Path):
+        a.read_fem(fem_file, fem_format, name, fem_converter=fem_converter)
     elif type(fem_file) is list:
         for i, f in enumerate(fem_file):
             fem_format_in = fem_format if fem_format is None else fem_format[i]
             name_in = name if name is None else name[i]
-            a.read_fem(f, fem_format_in, name_in)
+            a.read_fem(f, fem_format_in, name_in, fem_converter=fem_converter)
     else:
         raise ValueError(f'fem_file must be either string or list. Passed type was "{type(fem_file)}"')
 
@@ -75,6 +82,7 @@ __all__ = [
     "Material",
     "Shape",
     "Node",
+    "Placement",
     "PrimBox",
     "PrimCyl",
     "PrimExtrude",
@@ -85,7 +93,9 @@ __all__ = [
     "CurveRevolve",
     "LineSegment",
     "ArcSegment",
+    "Transform",
     "User",
     "Bolts",
     "Weld",
+    "fem",
 ]
