@@ -1,10 +1,11 @@
 from __future__ import annotations
 
 from .common import Amplitude, Csys
+from .concept import FEM
 from .constraints import Bc, Constraint, PredefinedField
 from .elements import Connector, Elem, Mass, Spring
 from .interactions import Interaction, InteractionProperty
-from .loads import Load, LoadCase
+from .loads import Load, LoadCase, LoadGravity, LoadPressure
 from .outputs import FieldOutput, HistOutput
 from .sections import ConnectorSection, FemSection
 from .sets import FemSet
@@ -17,12 +18,13 @@ __all__ = [
     "Csys",
     "InteractionProperty",
     "Interaction",
-    StepSteadyState,
-    StepEigen,
-    StepImplicit,
-    StepExplicit,
+    "StepSteadyState",
+    "StepEigen",
+    "StepImplicit",
+    "StepExplicit",
     "Surface",
     "Elem",
+    "FEM",
     "Connector",
     "Constraint",
     "PredefinedField",
@@ -32,6 +34,8 @@ __all__ = [
     "FieldOutput",
     "ConnectorSection",
     "Load",
+    "LoadGravity",
+    "LoadPressure",
     "LoadCase",
     "FemSection",
     "Spring",
