@@ -1,12 +1,14 @@
 import logging
 import os
 import pathlib
-from typing import List, Union
+from typing import TYPE_CHECKING, List, Union
 
 from ada.fem import StepEigen
 from ada.fem.concepts.eigenvalue import EigenDataSummary, EigenMode
 from ada.fem.formats.utils import DatFormatReader
-from ada.fem.results import Results
+
+if TYPE_CHECKING:
+    from ada.fem.results import Results
 
 
 def get_eigen_data(dat_file: Union[str, os.PathLike]) -> EigenDataSummary:
@@ -29,7 +31,7 @@ def get_eigen_data(dat_file: Union[str, os.PathLike]) -> EigenDataSummary:
     return EigenDataSummary(eigen_modes)
 
 
-def read_sesam_results(results: Results, file_ref: pathlib.Path, overwrite):
+def read_sesam_results(results: "Results", file_ref: pathlib.Path, overwrite):
     dat_file = (file_ref.parent / "SESTRA").with_suffix(".LIS")
     if dat_file.exists() and type(results.assembly.fem.steps[0]) == StepEigen:
         results.eigen_mode_data = get_eigen_data(dat_file)
